#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0
from collections import namedtuple

from common_utils_py.agreements.service_agreement_template import ServiceAgreementTemplate
from common_utils_py.agreements.service_types import ServiceTypes, ServiceTypesIndices
from common_utils_py.ddo.service import Service
from common_utils_py.did import did_to_id
from common_utils_py.utils.utilities import generate_prefixed_id

Agreement = namedtuple('Agreement', ('template', 'conditions'))


class ServiceAgreement(Service):
    """Class representing a Service Agreement."""
    AGREEMENT_TEMPLATE = 'serviceAgreementTemplate'
    SERVICE_CONDITIONS = 'conditions'

    def __init__(self, attributes, service_agreement_template, service_endpoint=None,
                 service_type=None, service_index=None, other_values=None):
        """

        :param attributes: dict of main attributes of the service. This should
            include `main` and optionally the `additionalInformation` section
        :param service_agreement_template: ServiceAgreementTemplate instance
        :param service_endpoint: str URL to use for requesting service defined in this agreement
        :param service_type: str like ServiceTypes.ASSET_ACCESS
        :param other_values: dict of other key/value that maybe added and will be kept as is.
        """
        self.service_agreement_template = service_agreement_template
        self._other_values = other_values or {}

        service_to_default_index = {
            ServiceTypes.ASSET_ACCESS: ServiceTypesIndices.DEFAULT_ACCESS_INDEX,
            ServiceTypes.CLOUD_COMPUTE: ServiceTypesIndices.DEFAULT_COMPUTING_INDEX
        }

        try:
            default_index = service_to_default_index[service_type]
        except KeyError:
            raise ValueError(f'The service_type {service_type} is not currently supported. Supported '
                             f'service types are {ServiceTypes.ASSET_ACCESS} and {ServiceTypes.CLOUD_COMPUTE}')

        service_index = service_index if service_index is not None else default_index
        Service.__init__(self, service_endpoint, service_type, attributes, other_values, service_index)

    @classmethod
    def from_json(cls, service_dict):
        """

        :param service_dict:
        :return:
        """
        service_endpoint, _type, _index, _attributes, service_dict = cls._parse_json(service_dict)
        template = ServiceAgreementTemplate(
            service_dict.pop('templateId'),
            _attributes['main']['name'],
            _attributes['main']['creator'],
            _attributes[cls.AGREEMENT_TEMPLATE]
        )

        return cls(
            _attributes,
            template,
            service_endpoint,
            _type,
            _index,
            service_dict
        )

    @classmethod
    def from_ddo(cls, service_type, ddo):
        """

        :param service_type: identifier of the service inside the asset DDO, str
        :param ddo:
        :return:
        """
        service_dict = ddo.get_service(service_type).as_dictionary()
        if not service_dict:
            raise ValueError(
                f'Service of type {service_type} is not found in this DDO.')

        return cls.from_json(service_dict)

    def as_dictionary(self):
        values = Service.as_dictionary(self)
        values[ServiceAgreementTemplate.TEMPLATE_ID_KEY] = self.template_id
        attributes = values[ServiceAgreement.SERVICE_ATTRIBUTES]
        attributes[ServiceAgreement.AGREEMENT_TEMPLATE] = self.service_agreement_template.template
        return values

    def init_conditions_values(self, did, contract_name_to_address):
        param_map = {
            '_documentId': did_to_id(did),
            '_amount': self.attributes['main']['price'],
            '_rewardAddress': contract_name_to_address['EscrowReward']
        }
        conditions = self.conditions[:]
        for cond in conditions:
            for param in cond.parameters:
                param.value = param_map.get(param.name, '')

            if cond.timeout > 0:
                cond.timeout = self.attributes['main']['timeout']

        self.service_agreement_template.set_conditions(conditions)

    def get_price(self):
        """
        Return the price from the conditions parameters.

        :return: Int
        """
        for cond in self.conditions:
            for p in cond.parameters:
                if p.name == '_amount':
                    return int(p.value)

    @property
    def service_endpoint(self):
        """

        :return:
        """
        return self._service_endpoint

    @property
    def agreement(self):
        """

        :return:
        """
        return Agreement(self.template_id, self.conditions[:])

    @property
    def template_id(self):
        """

        :return:
        """
        return self.service_agreement_template.template_id

    @property
    def conditions(self):
        """

        :return:
        """
        return self.service_agreement_template.conditions

    @property
    def condition_by_name(self):
        """

        :return:
        """
        return {cond.name: cond for cond in self.conditions}

    @property
    def conditions_params_value_hashes(self):
        """

        :return:
        """
        value_hashes = []
        for cond in self.conditions:
            value_hashes.append(cond.values_hash)

        return value_hashes

    @property
    def conditions_timeouts(self):
        """

        :return:
        """
        return [cond.timeout for cond in self.conditions]

    @property
    def conditions_timelocks(self):
        """

        :return:
        """
        return [cond.timelock for cond in self.conditions]

    @property
    def conditions_contracts(self):
        """

        :return:
        """
        return [cond.contract_name for cond in self.conditions]

    @staticmethod
    def generate_service_agreement_hash(template_id, values_hash_list, timelocks, timeouts,
                                        agreement_id, hash_function):
        """

        :param template_id:
        :param values_hash_list:
        :param timelocks:
        :param timeouts:
        :param agreement_id: id of the agreement, hex str
        :param hash_function: reference to function that will be used to compute the hash (sha3
        or similar)
        :return:
        """
        return hash_function(
            ['bytes32', 'bytes32[]', 'uint256[]', 'uint256[]', 'bytes32'],
            [template_id, values_hash_list, timelocks, timeouts, agreement_id]
        )

    @staticmethod
    def create_new_agreement_id():
        """

        :return:
        """
        return generate_prefixed_id()

    def generate_agreement_condition_ids(self, agreement_id, asset_id, consumer_address,
                                         publisher_address, keeper):
        """

        :param agreement_id: id of the agreement, hex str
        :param asset_id:
        :param consumer_address: ethereum account address of consumer, hex str
        :param publisher_address: ethereum account address of publisher, hex str
        :param keeper:
        :return:
        """
        lock_cond_id = keeper.lock_reward_condition.generate_id(
            agreement_id,
            self.condition_by_name['lockReward'].param_types,
            [keeper.escrow_reward_condition.address, self.get_price()]).hex()

        if self.type == ServiceTypes.ASSET_ACCESS:
            access_or_compute_id = keeper.access_secret_store_condition.generate_id(
                agreement_id,
                self.condition_by_name['accessSecretStore'].param_types,
                [asset_id, consumer_address]).hex()
        elif self.type == ServiceTypes.CLOUD_COMPUTE:
            access_or_compute_id = keeper.compute_execution_condition.generate_id(
                agreement_id,
                self.condition_by_name['computeExecution'].param_types,
                [asset_id, consumer_address]).hex()
        else:
            raise Exception(
                'Error generating the condition ids, the service_agreement type is not valid.')

        escrow_cond_id = keeper.escrow_reward_condition.generate_id(
            agreement_id,
            self.condition_by_name['escrowReward'].param_types,
            [self.get_price(), publisher_address, consumer_address,
             lock_cond_id, access_or_compute_id]).hex()

        return lock_cond_id, access_or_compute_id, escrow_cond_id

    def get_service_agreement_hash(
            self, agreement_id, asset_id, consumer_address, publisher_address, keeper):
        """Return the hash of the service agreement values to be signed by a consumer.

        :param agreement_id:id of the agreement, hex str
        :param asset_id:
        :param consumer_address: ethereum account address of consumer, hex str
        :param publisher_address: ethereum account address of publisher, hex str
        :param keeper:
        :return:
        """
        agreement_hash = ServiceAgreement.generate_service_agreement_hash(
            self.template_id,
            self.generate_agreement_condition_ids(
                agreement_id, asset_id, consumer_address, publisher_address, keeper),
            self.conditions_timelocks,
            self.conditions_timeouts,
            agreement_id,
            keeper.generate_multi_value_hash
        )
        return agreement_hash



#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

import json
import os
import pathlib

from ocean_keeper.utils import get_account

from common_utils_py.ddo.ddo import DDO

PUBLISHER_INDEX = 1
CONSUMER_INDEX = 0


def get_resource_path(dir_name, file_name):
    base = os.path.realpath(__file__).split(os.path.sep)[1:-1]
    if dir_name:
        return pathlib.Path(os.path.join(os.path.sep, *base, dir_name, file_name))
    else:
        return pathlib.Path(os.path.join(os.path.sep, *base, file_name))


def get_publisher_account():
    return get_account(0)


def get_consumer_account():
    return get_account(1)


def get_ddo_sample():
    return DDO(json_filename=get_resource_path('ddo', 'ddo_sa_sample.json'))


def log_event(event_name):
    def _process_event(event):
        print(f'Received event {event_name}: {event}')

    return _process_event


def get_metadata():
    path = get_resource_path('ddo', 'valid_metadata.json')
    assert path.exists(), f"{path} does not exist!"
    with open(path, 'r') as file_handle:
        metadata = file_handle.read()
    return json.loads(metadata)



#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0
import os

import pytest
from ocean_keeper.contract_handler import ContractHandler
from ocean_keeper.keeper import Keeper
from ocean_keeper.web3_provider import Web3Provider
from web3 import HTTPProvider, Web3

from common_utils_py.agreements.service_agreement import ServiceAgreement, ServiceTypes
from common_utils_py.aquarius import AquariusProvider
from common_utils_py.ddo.ddo import DDO
from common_utils_py.did import DID
from tests.resources.helper_functions import (
    get_consumer_account,
    get_ddo_sample,
    get_metadata,
    get_publisher_account
)


def get_aquarius_url():
    if os.getenv('AQUARIUS_URL'):
        return os.getenv('AQUARIUS_URL')
    return 'http://localhost:5000'


def get_keeper_url():
    if os.getenv('KEEPER_URL'):
        return os.getenv('KEEPER_URL')
    return 'http://localhost:8545'


@pytest.fixture(autouse=True)
def setup_all():
    Web3Provider.init_web3('http://localhost:8545')
    ContractHandler.set_artifacts_path(os.path.expanduser('~/.ocean/keeper-contracts/artifacts'))
    Keeper.get_instance()


@pytest.fixture
def publisher_account():
    return get_publisher_account()


@pytest.fixture
def consumer_account():
    return get_consumer_account()


@pytest.fixture
def aquarius():
    return AquariusProvider.get_aquarius(get_aquarius_url())


@pytest.fixture
def registered_ddo():
    return DDO()


@pytest.fixture
def web3_instance():
    return Web3(HTTPProvider(get_keeper_url()))


@pytest.fixture
def metadata():
    return get_metadata()


@pytest.fixture
def setup_agreements_environment():
    consumer_acc = get_consumer_account()
    publisher_acc = get_publisher_account()
    keeper = Keeper.get_instance()

    ddo = get_ddo_sample()
    ddo._did = DID.did({"0": "0x12341234"})
    keeper.did_registry.register(
        ddo.asset_id,
        checksum=Web3Provider.get_web3().toBytes(hexstr=ddo.asset_id),
        url='aquarius:5000',
        account=publisher_acc,
        providers=None
    )

    registered_ddo = ddo
    asset_id = registered_ddo.asset_id
    service_agreement = ServiceAgreement.from_ddo(ServiceTypes.ASSET_ACCESS, ddo)
    agreement_id = ServiceAgreement.create_new_agreement_id()
    price = service_agreement.get_price()
    (lock_cond_id,
     access_cond_id,
     escrow_cond_id) = service_agreement.generate_agreement_condition_ids(
            agreement_id, asset_id, consumer_acc.address, publisher_acc.address, keeper
    )

    return (
        keeper,
        publisher_acc,
        consumer_acc,
        agreement_id,
        asset_id,
        price,
        service_agreement,
        (lock_cond_id, access_cond_id, escrow_cond_id),
    )



"""
    Service Class
    To handle service items in a DDO record
"""

#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0
import copy
import json
import logging

# from ocean_commons.agreements.service_agreement import ServiceAgreement
# from ocean_commons.agreements.service_types import ServiceTypes

logger = logging.getLogger(__name__)


class Service:
    """Service class to create validate service in a DDO."""
    SERVICE_ENDPOINT = 'serviceEndpoint'
    SERVICE_TYPE = 'type'
    SERVICE_INDEX = 'index'
    SERVICE_ATTRIBUTES = 'attributes'

    def __init__(self, service_endpoint, service_type, attributes, other_values=None, index=None):
        """Initialize Service instance."""
        self._service_endpoint = service_endpoint
        self._type = service_type or ''
        self._index = index
        self._attributes = attributes or {}

        # assign the _values property to empty until they are used
        self._values = dict()
        self._reserved_names = {self.SERVICE_ENDPOINT, self.SERVICE_TYPE, self.SERVICE_INDEX}
        if other_values:
            for name, value in other_values.items():
                if name not in self._reserved_names:
                    self._values[name] = value

    @property
    def type(self):
        """
        Type of the service.

        :return: str
        """
        return self._type

    @property
    def index(self):
        """
        Identifier of the service inside the asset DDO

        :return: str
        """
        return self._index

    @property
    def service_endpoint(self):
        """
        Service endpoint.

        :return: String
        """
        return self._service_endpoint

    def set_service_endpoint(self, service_endpoint):
        """
        Update service endpoint. Needed to update after create did.

        :param service_endpoint: Service endpoint, str
        """
        self._service_endpoint = service_endpoint

    def values(self):
        """

        :return: array of values
        """
        return self._values.copy()

    @property
    def attributes(self):
        return self._attributes

    @property
    def main(self):
        return self._attributes['main']

    def update_value(self, name, value):
        """
        Update value in the array of values.

        :param name: Key of the value, str
        :param value: New value, str
        :return: None
        """
        if name not in self._reserved_names:
            self._values[name] = value

    def as_text(self, is_pretty=False):
        """Return the service as a JSON string."""
        values_dict = self.as_dictionary()
        if is_pretty:
            return json.dumps(values_dict, indent=4, separators=(',', ': '))

        return json.dumps(values_dict)

    def as_dictionary(self):
        """Return the service as a python dictionary."""
        attributes = {}
        for key, value in self._attributes.items():
            if isinstance(value, object) and hasattr(value, 'as_dictionary'):
                value = value.as_dictionary()
            elif isinstance(value, list):
                value = [v.as_dictionary() if hasattr(v, 'as_dictionary') else v for v in value]

            attributes[key] = value

        values = {
            self.SERVICE_TYPE: self._type,
            self.SERVICE_ENDPOINT: self._service_endpoint,
            self.SERVICE_ATTRIBUTES: attributes
        }
        if self._index is not None:
            values[self.SERVICE_INDEX] = self._index

        if self._values:
            values.update(self._values)

        return values

    @classmethod
    def _parse_json(cls, service_dict):
        sd = copy.deepcopy(service_dict)
        service_endpoint = sd.pop(cls.SERVICE_ENDPOINT, None)
        _type = sd.pop(cls.SERVICE_TYPE, None)
        _index = sd.pop(cls.SERVICE_INDEX, None)
        _attributes = sd.pop(cls.SERVICE_ATTRIBUTES, None)
        if not service_endpoint:
            logger.error(
                'Service definition in DDO document is missing the "serviceEndpoint" key/value.')
            raise IndexError

        if not _type:
            logger.error('Service definition in DDO document is missing the "type" key/value.')
            raise IndexError

        return service_endpoint, _type, _index, _attributes, sd

    @classmethod
    def from_json(cls, service_dict):
        """Create a service object from a JSON string."""
        service_endpoint, _type, _index, _attributes, sd = cls._parse_json(service_dict)
        return cls(
            service_endpoint,
            _type,
            _attributes,
            sd,
            _index
        )




#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

KEY_PAIR_MODULUS_BIT = 1024

DID_DDO_CONTEXT_URL = 'https://w3id.org/did/v1'

PROOF_TYPE = 'DDOIntegritySignature'



#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

from common_utils_py.agreements.service_agreement import ServiceAgreement
from common_utils_py.agreements.service_agreement_template import ServiceAgreementTemplate
from common_utils_py.agreements.service_types import ServiceTypes, ServiceTypesIndices
from common_utils_py.agreements.utils import get_sla_template
from common_utils_py.ddo.service import Service


class ServiceDescriptor(object):
    """Tuples of length 2. The first item must be one of ServiceTypes and the second
    item is a dict of parameters and values required by the service"""

    @staticmethod
    def metadata_service_descriptor(attributes, service_endpoint):
        """
        Metadata service descriptor.

        :param attributes: conforming to the Metadata accepted by Ocean Protocol, dict
        :param service_endpoint: identifier of the service inside the asset DDO, str
        :return: Service descriptor.
        """
        return (ServiceTypes.METADATA,
                {'attributes': attributes, 'serviceEndpoint': service_endpoint})

    @staticmethod
    def authorization_service_descriptor(service_endpoint):
        """
        Authorization service descriptor.

        :param service_endpoint: identifier of the service inside the asset DDO, str
        :return: Service descriptor.
        """
        return (ServiceTypes.AUTHORIZATION,
                {'attributes': {'main': {}}, 'serviceEndpoint': service_endpoint})

    @staticmethod
    def access_service_descriptor(attributes, service_endpoint, template_id):
        """
        Access service descriptor.

        :param attributes: attributes of the access service, dict
        :param service_endpoint: identifier of the service inside the asset DDO, str
        :param template_id:
        :return: Service descriptor.
        """
        return (
            ServiceTypes.ASSET_ACCESS,
            {'attributes': attributes,
             'serviceEndpoint': service_endpoint,
             'templateId': template_id}
        )

    @staticmethod
    def compute_service_descriptor(attributes, service_endpoint, template_id):
        """
        Compute service descriptor.

        :param attributes: attributes of the compute service, dict
        :param service_endpoint: identifier of the service inside the asset DDO, str
        :param template_id:
        :return: Service descriptor.
        """
        return (
            ServiceTypes.CLOUD_COMPUTE,
            {'attributes': attributes,
             'serviceEndpoint': service_endpoint,
             'templateId': template_id}
        )


class ServiceFactory(object):
    """Factory class to create Services."""

    @staticmethod
    def build_services(service_descriptors):
        """
        Build a list of services.

        :param service_descriptors: List of tuples of length 2. The first item must be one of
        ServiceTypes
        and the second item is a dict of parameters and values required by the service
        :return: List of Services
        """
        services = []
        for i, service_desc in enumerate(service_descriptors):
            service = ServiceFactory.build_service(service_desc)
            # set index for each service
            service.update_value(ServiceAgreement.SERVICE_INDEX, int(i))
            services.append(service)

        return services

    @staticmethod
    def build_service(service_descriptor):
        """
        Build a service.

        :param service_descriptor: Tuples of length 2. The first item must be one of ServiceTypes
        and the second item is a dict of parameters and values required by the service
        :return: Service
        """
        assert isinstance(service_descriptor, tuple) and len(
            service_descriptor) == 2, 'Unknown service descriptor format.'
        service_type, kwargs = service_descriptor
        if service_type == ServiceTypes.METADATA:
            return ServiceFactory.build_metadata_service(
                kwargs['attributes'],
                kwargs['serviceEndpoint']
            )

        elif service_type == ServiceTypes.AUTHORIZATION:
            return ServiceFactory.build_authorization_service(
                kwargs['attributes'],
                kwargs['serviceEndpoint']
            )

        elif service_type == ServiceTypes.ASSET_ACCESS:
            return ServiceFactory.build_access_service(
                kwargs['attributes'],
                kwargs['serviceEndpoint'],
                kwargs['templateId']
            )
        elif service_type == ServiceTypes.CLOUD_COMPUTE:
            return ServiceFactory.build_compute_service(
                kwargs['attributes'],
                kwargs['serviceEndpoint'],
                kwargs['templateId']
            )
        raise ValueError(f'Unknown service type {service_type}')

    @staticmethod
    def build_metadata_service(metadata, service_endpoint):
        """
        Build a metadata service.

        :param metadata: conforming to the Metadata accepted by Ocean Protocol, dict
        :param service_endpoint: identifier of the service inside the asset DDO, str
        :return: Service
        """
        return Service(service_endpoint,
                       ServiceTypes.METADATA,
                       attributes=metadata,
                       index=ServiceTypesIndices.DEFAULT_METADATA_INDEX
                       )

    @staticmethod
    def build_authorization_service(attributes, service_endpoint):
        """
        Build an authorization service.

        :param attributes: attributes of authorization service, dict
        :param service_endpoint: identifier of the service inside the asset DDO, str
        :return: Service
        """
        return Service(service_endpoint, ServiceTypes.AUTHORIZATION,
                       attributes=attributes,
                       index=ServiceTypesIndices.DEFAULT_AUTHORIZATION_INDEX)

    @staticmethod
    def build_access_service(attributes, service_endpoint, template_id):
        """
        Build an authorization service.

        :param attributes: attributes of access service, dict
        :param service_endpoint: identifier of the service inside the asset DDO, str
        :param template_id: hex str the ethereum smart contract address of the
            service agreement template contract.
        :return: ServiceAgreement instance
        """
        sla_template_dict = get_sla_template()
        sla_template = ServiceAgreementTemplate(
            template_id, 'dataAssetAccessServiceAgreement',
            attributes['main']['creator'], sla_template_dict
        )
        return ServiceAgreement(
            attributes,
            sla_template,
            service_endpoint,
            ServiceTypes.ASSET_ACCESS,
            ServiceTypesIndices.DEFAULT_ACCESS_INDEX
        )

    @staticmethod
    def build_compute_service(attributes, service_endpoint, template_id):
        """
        Build an authorization service.

        :param attributes: attributes of compute service, dict
        :param service_endpoint: identifier of the service inside the asset DDO, str
        :param template_id: hex str the ethereum smart contract address of the
            service agreement template contract.
        :return: ServiceAgreement instance
        """
        sla_template_dict = get_sla_template(ServiceTypes.CLOUD_COMPUTE)
        sla_template = ServiceAgreementTemplate(
            template_id, 'dataComputeServiceAgreement',
            attributes['main']['creator'], sla_template_dict
        )
        return ServiceAgreement(
            attributes,
            sla_template,
            service_endpoint,
            ServiceTypes.CLOUD_COMPUTE,
            ServiceTypesIndices.DEFAULT_COMPUTING_INDEX
        )



#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

from web3 import Web3

from common_utils_py.utils import utilities
from tests.resources.tiers import e2e_test


@e2e_test
def test_convert():
    input_text = "my text"
    text_bytes = utilities.convert_to_bytes(Web3, input_text)
    print("output %s" % utilities.convert_to_string(Web3, text_bytes))
    assert input_text == utilities.convert_to_text(Web3, text_bytes)



#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

import os

import pytest


def unit_test(test):
    """
    Mark all test (suites) with this decorator that test isolated functions classes optionally
    with mocked out dependencies.
    Unit tests should live next to the file there testing with a *_test.py suffix,
    e.g. `foo_test.py` next to `foo.py`.
    """

    return test  # Always run all unit tests for now


def integration_test(test):
    """
    Mark all test (suites) with this decorator that test interactions between multiple classes,
    but don't touch external services.
    Integration tests should live in the `common_utils_py/test/integration` folder and
    `common_utils_py/<package>/test` folders of individual sub-packages.
    """

    return pytest.mark.skip(reason='Integration tests not enabled')(test) if not should_run_test(
        'integration') else test


def e2e_test(test):
    """
    Mark all test (suites) with this decorator that test interactions between multiple classes
    and external services.
    End-to-end tests should live in the `common_utils_py/test/e2e` and `common_utils_py/examples` folders.
    """

    return pytest.mark.skip(reason='End to end tests not enabled')(test) if not should_run_test(
        'e2e') else test


def should_run_test(test_tier, active_tier=None):
    tiers = ['unit', 'integration', 'e2e']
    active_tier_index = tiers.index(active_tier or _get_active_test_tier())
    test_tier_index = tiers.index(test_tier)
    return test_tier_index <= active_tier_index


def _get_active_test_tier():
    return os.getenv('TEST_TIER', 'e2e')



"""Exceptions for common_utils """


#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0


class OceanInvalidContractAddress(Exception):
    """Raised when an invalid address is passed to the contract loader."""


class OceanDIDUnknownValueType(Exception):
    """Raised when a requested DID or a DID in the chain cannot be found."""


class OceanDIDAlreadyExist(Exception):
    """Raised when a requested DID is already published in OceanDB."""


class OceanInvalidMetadata(Exception):
    """Raised when some value in the metadata is invalid."""


class OceanInvalidServiceAgreementSignature(Exception):
    """Raised when the SLA signature is not valid."""


class OceanServiceAgreementExists(Exception):
    """Raised when the SLA already exists."""


class OceanInitializeServiceAgreementError(Exception):
    """Error on invoking purchase endpoint"""


class OceanEncryptAssetUrlsError(Exception):
    """Error invoking the encrypt endpoint"""


class OceanServiceConsumeError(Exception):
    """ Error invoking a purchase endpoint"""


class OceanInvalidAgreementTemplate(Exception):
    """ Error when agreement template is not valid or not approved"""



"""Agreements module."""


#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

class ServiceTypes:
    """Types of Service allowed in ocean protocol DDO services."""
    AUTHORIZATION = 'authorization'
    METADATA = 'metadata'
    ASSET_ACCESS = 'access'
    CLOUD_COMPUTE = 'compute'


class ServiceTypesIndices:
    DEFAULT_METADATA_INDEX = 0
    DEFAULT_PROVENANCE_INDEX = 1
    DEFAULT_AUTHORIZATION_INDEX = 2
    DEFAULT_ACCESS_INDEX = 3
    DEFAULT_COMPUTING_INDEX = 4



COMPUTE_SLA_TEMPLATE = {
  "type": "compute",
  "templateId": "",
  "name": "dataAssetComputeExecutionAgreement",
  "description": "",
  "creator": "",
  "serviceAgreementTemplate": {
    "contractName": "EscrowComputeExecutionTemplate",
    "events": [
      {
        "name": "AgreementCreated",
        "actorType": "consumer",
        "handler": {
          "moduleName": "EscrowComputeExecutionTemplate",
          "functionName": "fulfillLockRewardCondition",
          "version": "0.1"
        }
      }
    ],
    "fulfillmentOrder": [
      "lockReward.fulfill",
      "computeExecution.fulfill",
      "escrowReward.fulfill"
    ],
    "conditionDependency": {
      "lockReward": [],
      "computeExecution": [],
      "escrowReward": [
        "lockReward",
        "computeExecution"
      ]
    },
    "conditions": [
        {
            "name": "lockReward",
            "timelock": 0,
            "timeout": 0,
            "contractName": "LockRewardCondition",
            "functionName": "fulfill",
            "parameters": [
                {
                    "name": "_rewardAddress",
                    "type": "address",
                    "value": "{contract.EscrowReward.address}"
                },
                {
                    "name": "_amount",
                    "type": "uint256",
                    "value": "{parameter.price}"
                }
            ],
            "events": [
                {
                    "name": "Fulfilled",
                    "actorType": "publisher",
                    "handler": {
                        "moduleName": "lockRewardCondition",
                        "functionName": "fulfillComputeExecutionCondition",
                        "version": "0.1"
                    }
                }
            ]
        },
        {
            "name": "computeExecution",
            "timelock": 0,
            "timeout": 0,
            "contractName": "ComputeExecutionCondition",
            "functionName": "fulfill",
            "parameters": [
                {
                    "name": "_documentId",
                    "type": "bytes32",
                    "value": "{parameter.assetId}"
                },
                {
                    "name": "_grantee",
                    "type": "address",
                    "value": ""
                }
            ],
            "events": [
                {
                    "name": "Fulfilled",
                    "actorType": "publisher",
                    "handler": {
                        "moduleName": "accessSecretStore",
                        "functionName": "fulfillEscrowRewardCondition",
                        "version": "0.1"
                    }
                },
                {
                    "name": "TimedOut",
                    "actorType": "consumer",
                    "handler": {
                        "moduleName": "computeExecution",
                        "functionName": "fulfillEscrowRewardCondition",
                        "version": "0.1"
                    }
                }
            ]
        },
        {
            "name": "escrowReward",
            "timelock": 0,
            "timeout": 0,
            "contractName": "EscrowReward",
            "functionName": "fulfill",
            "parameters": [
                {
                    "name": "_amount",
                    "type": "uint256",
                    "value": "{parameter.price}"
                },
                {
                    "name": "_receiver",
                    "type": "address",
                    "value": ""
                },
                {
                    "name": "_sender",
                    "type": "address",
                    "value": ""
                },
                {
                    "name": "_lockCondition",
                    "type": "bytes32",
                    "value": "{contract.LockRewardCondition.address}"
                },
                {
                    "name": "_releaseCondition",
                    "type": "bytes32",
                    "value": "{contract.ComputeExecutionCondition.address}"
                }
            ],
            "events": [
                {
                    "name": "Fulfilled",
                    "actorType": "publisher",
                    "handler": {
                        "moduleName": "escrowRewardCondition",
                        "functionName": "verifyRewardTokens",
                        "version": "0.1"
                    }
                }
            ]
        }
    ]
  }
}


#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

import logging
import secrets

import pytest
from ocean_keeper import Keeper
from ocean_keeper.exceptions import (
    OceanDIDNotFound,
)
from web3 import Web3

from common_utils_py.ddo.ddo import DDO
from common_utils_py.did import DID, did_to_id
from common_utils_py.did_resolver.did_resolver import (
    DIDResolver,
)
from tests.resources.helper_functions import get_resource_path
from tests.resources.tiers import e2e_test

logger = logging.getLogger()


def keeper():
    return Keeper.get_instance()


@e2e_test
def test_did_resolver_library(publisher_account, aquarius):
    did_registry = keeper().did_registry
    checksum_test = Web3.sha3(text='checksum')
    value_test = aquarius.root_url

    did_resolver = DIDResolver(keeper().did_registry)

    sample_ddo_path = get_resource_path('ddo', 'ddo_sample1.json')
    assert sample_ddo_path.exists(), "{} does not exist!".format(sample_ddo_path)
    asset1 = DDO(json_filename=sample_ddo_path)
    asset1._did = DID.did({"0": "0x1098098"})
    did_registry.register(asset1.asset_id, checksum_test, url=value_test, account=publisher_account)
    aquarius.publish_asset_ddo(asset1)

    did_resolved = did_resolver.resolve(asset1.did)
    assert did_resolved
    assert did_resolved.did == asset1.did

    with pytest.raises(ValueError):
        did_resolver.resolve(asset1.asset_id)

    aquarius.retire_asset_ddo(asset1.did)


@e2e_test
def test_did_not_found():
    did_resolver = DIDResolver(keeper().did_registry)
    did_id = secrets.token_hex(32)
    did_id_bytes = Web3.toBytes(hexstr=did_id)
    with pytest.raises(OceanDIDNotFound):
        did_resolver.resolve(did_id_bytes)


@e2e_test
def test_get_resolve_url(aquarius, publisher_account):
    register_account = publisher_account
    did_registry = keeper().did_registry
    did = DID.did({"0": "0x1"})
    asset_id = did_to_id(did)
    value_test = aquarius.root_url
    did_resolver = DIDResolver(keeper().did_registry)
    did_registry.register(asset_id, b'test', url=value_test, account=register_account)
    url = did_resolver.get_resolve_url(Web3.toBytes(hexstr=asset_id))
    assert url == value_test


@e2e_test
def test_get_resolve_multiple_urls(publisher_account):
    register_account = publisher_account
    did_registry = keeper().did_registry
    did = DID.did({"0": "0x1"})
    did2 = DID.did({"0": "0x2"})
    did3 = DID.did({"0": "0x3"})
    did4 = DID.did({"0": "0x4"})
    did5 = DID.did({"0": "0x5"})
    did6 = DID.did({"0": "0x6"})
    did7 = DID.did({"0": "0x7"})
    did8 = DID.did({"0": "0x8"})
    did9 = DID.did({"0": "0x9"})
    did10 = DID.did({"0": "0x10"})
    value_test = 'http://localhost:5000'
    value_test2 = 'http://localhost:5001'
    value_test3 = 'http://localhost:5002'
    value_test4 = 'http://localhost:5003'
    value_test5 = 'http://localhost:5004'
    value_test6 = 'http://localhost:5005'
    value_test7 = 'http://localhost:5006'
    value_test8 = 'http://localhost:5007'
    value_test9 = 'http://localhost:5008'
    value_test10 = 'http://localhost:5009'
    did_id = did_to_id(did)
    did_id2 = did_to_id(did2)
    did_id3 = did_to_id(did3)
    did_id4 = did_to_id(did4)
    did_id5 = did_to_id(did5)
    did_id6 = did_to_id(did6)
    did_id7 = did_to_id(did7)
    did_id8 = did_to_id(did8)
    did_id9 = did_to_id(did9)
    did_id10 = did_to_id(did10)
    did_resolver = DIDResolver(keeper().did_registry)
    did_registry.register(did_id, b'test', url=value_test, account=register_account)
    did_registry.register(did_id2, b'test', url=value_test2, account=register_account)
    did_registry.register(did_id3, b'test', url=value_test3, account=register_account)
    did_registry.register(did_id4, b'test', url=value_test4, account=register_account)
    did_registry.register(did_id5, b'test', url=value_test5, account=register_account)
    did_registry.register(did_id6, b'test', url=value_test6, account=register_account)
    did_registry.register(did_id7, b'test', url=value_test7, account=register_account)
    did_registry.register(did_id8, b'test', url=value_test8, account=register_account)
    did_registry.register(did_id9, b'test', url=value_test9, account=register_account)
    did_registry.register(did_id10, b'test', url=value_test10, account=register_account)
    url = did_resolver.get_resolve_url(Web3.toBytes(hexstr=did_id))
    url2 = did_resolver.get_resolve_url(Web3.toBytes(hexstr=did_id2))
    url3 = did_resolver.get_resolve_url(Web3.toBytes(hexstr=did_id3))
    url4 = did_resolver.get_resolve_url(Web3.toBytes(hexstr=did_id4))
    url5 = did_resolver.get_resolve_url(Web3.toBytes(hexstr=did_id5))
    url6 = did_resolver.get_resolve_url(Web3.toBytes(hexstr=did_id6))
    url7 = did_resolver.get_resolve_url(Web3.toBytes(hexstr=did_id7))
    url8 = did_resolver.get_resolve_url(Web3.toBytes(hexstr=did_id8))
    url9 = did_resolver.get_resolve_url(Web3.toBytes(hexstr=did_id9))
    url10 = did_resolver.get_resolve_url(Web3.toBytes(hexstr=did_id10))
    assert url == value_test
    assert url2 == value_test2
    assert url3 == value_test3
    assert url4 == value_test4
    assert url5 == value_test5
    assert url6 == value_test6
    assert url7 == value_test7
    assert url8 == value_test8
    assert url9 == value_test9
    assert url10 == value_test10


@e2e_test
def test_get_did_not_valid():
    did_resolver = DIDResolver(keeper().did_registry)
    with pytest.raises(TypeError):
        did_resolver.get_resolve_url('not valid')



ACCESS_SLA_TEMPLATE = {
  "type": "access",
  "templateId": "",
  "name": "dataAssetAccessServiceAgreement",
  "description": "This service agreement defines the flow for accessing a data asset on the ocean network. Any file or bundle of files can be access using this service agreement",
  "creator": "",
  "serviceAgreementTemplate": {
    "contractName": "EscrowAccessSecretStoreTemplate",
    "events": [
      {
        "name": "AgreementCreated",
        "actorType": "consumer",
        "handler": {
          "moduleName": "escrowAccessSecretStoreTemplate",
          "functionName": "fulfillLockRewardCondition",
          "version": "0.1"
        }
      }
    ],
    "fulfillmentOrder": [
      "lockReward.fulfill",
      "accessSecretStore.fulfill",
      "escrowReward.fulfill"
    ],
    "conditionDependency": {
      "lockReward": [],
      "accessSecretStore": [],
      "escrowReward": [
        "lockReward",
        "accessSecretStore"
      ]
    },
    "conditions": [
      {
        "name": "lockReward",
        "timelock": 0,
        "timeout": 0,
        "contractName": "LockRewardCondition",
        "functionName": "fulfill",
        "parameters": [
          {
            "name": "_rewardAddress",
            "type": "address",
            "value": ""
          },
          {
            "name": "_amount",
            "type": "uint256",
            "value": ""
          }
        ],
        "events": [
          {
            "name": "Fulfilled",
            "actorType": "publisher",
            "handler": {
              "moduleName": "lockRewardCondition",
              "functionName": "fulfillAccessSecretStoreCondition",
              "version": "0.1"
            }
          }
        ]
      },
      {
        "name": "accessSecretStore",
        "timelock": 0,
        "timeout": 0,
        "contractName": "AccessSecretStoreCondition",
        "functionName": "fulfill",
        "parameters": [
          {
            "name": "_documentId",
            "type": "bytes32",
            "value": ""
          },
          {
            "name": "_grantee",
            "type": "address",
            "value": ""
          }
        ],
        "events": [
          {
            "name": "Fulfilled",
            "actorType": "publisher",
            "handler": {
              "moduleName": "accessSecretStore",
              "functionName": "fulfillEscrowRewardCondition",
              "version": "0.1"
            }
          },
          {
            "name": "TimedOut",
            "actorType": "consumer",
            "handler": {
              "moduleName": "accessSecretStore",
              "functionName": "fulfillEscrowRewardCondition",
              "version": "0.1"
            }
          }
        ]
      },
      {
        "name": "escrowReward",
        "timelock": 0,
        "timeout": 0,
        "contractName": "EscrowReward",
        "functionName": "fulfill",
        "parameters": [
          {
            "name": "_amount",
            "type": "uint256",
            "value": ""
          },
          {
            "name": "_receiver",
            "type": "address",
            "value": ""
          },
          {
            "name": "_sender",
            "type": "address",
            "value": ""
          },
          {
            "name": "_lockCondition",
            "type": "bytes32",
            "value": ""
          },
          {
            "name": "_releaseCondition",
            "type": "bytes32",
            "value": ""
          }
        ],
        "events": [
          {
            "name": "Fulfilled",
            "actorType": "publisher",
            "handler": {
              "moduleName": "escrowRewardCondition",
              "functionName": "verifyRewardTokens",
              "version": "0.1"
            }
          }
        ]
      }
    ]
  }
}


"""
    Test did_lib
"""
#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

import json

import pytest
from ocean_keeper import Keeper

from common_utils_py.agreements.service_agreement import ServiceTypes, ServiceAgreement
from common_utils_py.agreements.service_factory import ServiceDescriptor, ServiceFactory
from common_utils_py.ddo.ddo import DDO
from common_utils_py.ddo.public_key_base import PublicKeyBase
from common_utils_py.ddo.public_key_rsa import PUBLIC_KEY_TYPE_ETHEREUM_ECDSA, PUBLIC_KEY_TYPE_RSA
from common_utils_py.did import DID, did_to_id_bytes
from common_utils_py.utils.utilities import checksum
from tests.resources.helper_functions import (get_ddo_sample, get_publisher_account,
                                              get_resource_path)
from tests.resources.tiers import unit_test

TEST_SERVICE_TYPE = 'ocean-meta-storage'
TEST_SERVICE_URL = 'http://localhost:8005'


def _get_sample_ddo(name):
    sample_ddo_path = get_resource_path('ddo', name)
    assert sample_ddo_path.exists(), f'{sample_ddo_path} does not exist!'
    with open(sample_ddo_path) as f:
        sample_ddo_json_dict = json.load(f)

    return sample_ddo_json_dict


def test_create_ddo(metadata):
    pub_acc = get_publisher_account()
    ddo = DDO()
    ddo.add_service(ServiceTypes.METADATA, 'http://myaquarius.com', values=metadata, index='0')
    checksums = dict()
    for service in ddo.services:
        checksums[str(service.index)] = checksum(service.main)
    ddo.add_proof(checksums, pub_acc)
    did = ddo.assign_did(DID.did(ddo.proof['checksum']))
    ddo.proof['signatureValue'] = Keeper.sign_hash(did_to_id_bytes(did), pub_acc)
    ddo.add_public_key(did, pub_acc.address)
    ddo.add_authentication(did, PUBLIC_KEY_TYPE_RSA)


@unit_test
def test_creating_ddo_from_scratch():
    # create an empty ddo
    ddo = DDO()
    assert ddo.did is None
    assert ddo.asset_id is None
    assert ddo.created is not None

    did = DID.did({"0": "0x99999999999999999"})
    ddo.assign_did(did)
    assert ddo.did == did

    pub_acc = get_publisher_account()

    ddo.add_service(TEST_SERVICE_TYPE, TEST_SERVICE_URL)

    # add a proof to the first public_key/authentication
    ddo.add_proof('checksum', pub_acc)
    ddo_text_proof = ddo.as_text()
    assert ddo_text_proof

    pub_acc = get_publisher_account()
    assert not ddo.public_keys
    ddo.add_public_key(did, pub_acc.address)
    assert len(ddo.public_keys) == 1
    assert ddo.get_public_key(0) == ddo.public_keys[0]
    with pytest.raises(IndexError):
        ddo.get_public_key(1)

    assert ddo.get_public_key(did) == ddo.public_keys[0]
    assert ddo.get_public_key('0x32233') is None

    assert not ddo.authentications
    ddo.add_authentication(did, '')
    assert len(ddo.authentications) == 1


@unit_test
def test_create_auth_from_json():
    auth = {'publicKey': '0x00000', 'type': 'auth-type', 'nothing': ''}
    assert (
        DDO.create_authentication_from_json(auth)
        ==
        {'publicKey': '0x00000', 'type': 'auth-type'}
    )
    with pytest.raises(ValueError):
        DDO.create_authentication_from_json({'type': 'auth-type'})


@unit_test
def test_create_public_key_from_json():
    pkey = {'id': 'pkeyid', 'type': 'keytype', 'owner': '0x00009'}
    pub_key_inst = DDO.create_public_key_from_json(pkey)
    assert isinstance(pub_key_inst, PublicKeyBase)
    assert pub_key_inst.get_id() == pkey['id']
    assert pub_key_inst.get_type() == PUBLIC_KEY_TYPE_ETHEREUM_ECDSA
    assert pub_key_inst.get_owner() == pkey['owner']

    pub_key_inst = DDO.create_public_key_from_json(
        {'type': PUBLIC_KEY_TYPE_RSA}
    )
    assert pub_key_inst.get_id() == ''
    assert pub_key_inst.get_type() == PUBLIC_KEY_TYPE_RSA
    assert pub_key_inst.get_owner() is None


@unit_test
def test_load_ddo_json():
    # TODO: Fix
    sample_ddo_json_dict = _get_sample_ddo('ddo_sample1.json')
    sample_ddo_json_string = json.dumps(sample_ddo_json_dict)

    this_ddo = DDO(json_text=sample_ddo_json_string)
    service = this_ddo.get_service(ServiceTypes.METADATA)
    assert service
    assert service.type == ServiceTypes.METADATA
    assert service.attributes
    assert this_ddo.as_dictionary() == sample_ddo_json_dict, \
        f'ddo seems to have changed after being loaded into DDO object: \n' \
        f'{this_ddo.as_text()} vs. {sample_ddo_json_string}'


@unit_test
def test_ddo_dict():
    sample_ddo_path = get_resource_path('ddo', 'ddo_sample1.json')
    assert sample_ddo_path.exists(), f'{sample_ddo_path} does not exist!'

    ddo1 = DDO(json_filename=sample_ddo_path)
    assert len(ddo1.public_keys) == 3
    assert ddo1.did == 'did:op:0c184915b07b44c888d468be85a9b28253e80070e5294b1aaed81c2f0264e429'


@unit_test
def test_find_service():
    ddo = get_ddo_sample()
    service = ddo.get_service_by_index(0)
    assert service and service.type == ServiceTypes.METADATA, 'Failed to find service by integer ' \
                                                              'id.'
    service = ddo.get_service_by_index('0')
    assert service and service.type == ServiceTypes.METADATA, 'Failed to find service by str(int)' \
                                                              ' id.'

    service = ddo.get_service_by_index(3)
    assert service and service.type == ServiceTypes.ASSET_ACCESS, 'Failed to find service by ' \
                                                                  'integer id.'
    service = ddo.get_service_by_index('3')
    assert service and service.type == ServiceTypes.ASSET_ACCESS, 'Failed to find service by str(' \
                                                                  'int) id.'

    service = ddo.get_service(ServiceTypes.ASSET_ACCESS)
    assert service and service.type == ServiceTypes.ASSET_ACCESS, 'Failed to find service by id ' \
                                                                  'using service type.'
    assert service.index == 3, 'index not as expected.'

    service = ddo.get_service(ServiceTypes.METADATA)
    assert service and service.type == ServiceTypes.METADATA, 'Failed to find service by id using ' \
                                                              '' \
                                                              'service ' \
                                                              'type.'
    assert service.index == 0, 'index not as expected.'


@unit_test
def test_service_factory():
    ddo = DDO(dictionary=_get_sample_ddo('ddo_sample1.json'))
    type_to_service = {s.type: s for s in ddo.services}
    metadata = ddo.metadata

    md_descriptor = ServiceDescriptor.metadata_service_descriptor(metadata, type_to_service[ServiceTypes.METADATA].service_endpoint)
    access_service = type_to_service[ServiceTypes.ASSET_ACCESS]
    access_descriptor = ServiceDescriptor.access_service_descriptor(access_service.attributes, access_service.service_endpoint, access_service.template_id)
    compute_descriptor = ServiceDescriptor.compute_service_descriptor(access_service.attributes, access_service.service_endpoint, access_service.template_id)

    services = ServiceFactory.build_services([md_descriptor, access_descriptor, compute_descriptor])
    assert len(services) == 3
    assert services[0].type == ServiceTypes.METADATA
    assert services[1].type == ServiceTypes.ASSET_ACCESS
    assert services[2].type == ServiceTypes.CLOUD_COMPUTE

    keeper = Keeper.get_instance()
    name_to_address = {name: i.address for name, i in keeper.contract_name_to_instance.items()}
    s = services[1]
    s.init_conditions_values(ddo.did, name_to_address)
    s = services[2]
    s.init_conditions_values(ddo.did, name_to_address)

    assert isinstance(services[1], ServiceAgreement)
    assert isinstance(services[2], ServiceAgreement)



"""Aquarius module."""

#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

from .aquarius import Aquarius


class AquariusProvider:
    """Provides the Aquarius instance."""
    _aquarius_class = Aquarius

    @staticmethod
    def get_aquarius(url):
        """ Get an Aquarius instance."""
        return AquariusProvider._aquarius_class(url)

    @staticmethod
    def set_aquarius_class(aquarius_class):
        """
         Set an Aquarius class

        :param aquarius_class: Aquarius or similar compatible class
        """
        AquariusProvider._aquarius_class = aquarius_class



#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

from ocean_keeper.utils import generate_multi_value_hash
from web3 import Web3

from common_utils_py.agreements.service_agreement import ServiceAgreement, ServiceTypes
from tests.resources.helper_functions import (
    get_ddo_sample,
    log_event
)
from tests.resources.tiers import e2e_test


@e2e_test
def test_escrow_access_secret_store_template_flow(setup_agreements_environment):
    """Test the agreement flow according to the EscrowAccessSecretStoreTemplate."""
    (
        keeper,
        publisher_acc,
        consumer_acc,
        agreement_id,
        asset_id,
        price,
        service_agreement,
        (lock_cond_id, access_cond_id, escrow_cond_id),

    ) = setup_agreements_environment

    print('creating agreement:'
          'agrId: ', agreement_id,
          'asset_id', asset_id,
          '[lock_cond_id, access_cond_id, escrow_cond_id]',
          [lock_cond_id, access_cond_id, escrow_cond_id],
          'tlocks', service_agreement.conditions_timelocks,
          'touts', service_agreement.conditions_timeouts,
          'consumer', consumer_acc.address,
          'publisher', publisher_acc.address
          )

    assert 'access' in keeper.template_manager.SERVICE_TO_TEMPLATE_NAME
    name = keeper.template_manager.SERVICE_TO_TEMPLATE_NAME.get('access')
    template_id = keeper.template_manager.create_template_id(name)
    assert keeper.template_manager.is_template_approved(template_id), 'Template is not approved.'
    assert keeper.did_registry.get_block_number_updated(asset_id) > 0, 'asset id not registered'
    success = keeper.agreement_manager.create_agreement(
        agreement_id,
        asset_id,
        template_id,
        [lock_cond_id, access_cond_id, escrow_cond_id],
        service_agreement.conditions_timelocks,
        service_agreement.conditions_timeouts,
        [consumer_acc.address, publisher_acc.address],
        consumer_acc
    )
    print('create agreement: ', success)
    assert success, f'createAgreement failed {success}'
    event = keeper.agreement_manager.subscribe_agreement_created(
        agreement_id,
        10,
        log_event(keeper.agreement_manager.AGREEMENT_CREATED_EVENT),
        (),
        wait=True
    )
    assert event, 'no event for AgreementCreated '

    # Verify condition types (condition contracts)
    agreement = keeper.agreement_manager.get_agreement(agreement_id)
    assert agreement.did == asset_id, ''
    template_values = keeper.template_manager.get_template(template_id)
    cond_types = template_values.condition_types
    for i, cond_id in enumerate(agreement.condition_ids):
        cond = keeper.condition_manager.get_condition(cond_id)
        assert cond.type_ref == cond_types[i]
        assert int(cond.state) == 1

    # Give consumer some tokens
    keeper.dispenser.request_vodkas(price, consumer_acc)
    # Fulfill lock_reward_condition
    pub_token_balance = keeper.token.get_token_balance(publisher_acc.address)
    starting_balance = keeper.token.get_token_balance(keeper.escrow_reward_condition.address)
    approved = keeper.token.token_approve(keeper.lock_reward_condition.address, price, consumer_acc)
    assert approved is True, 'token approval failed.'
    tx_hash = keeper.lock_reward_condition.fulfill(
        agreement_id, keeper.escrow_reward_condition.address, price, consumer_acc)
    keeper.lock_reward_condition.get_tx_receipt(tx_hash)
    event = keeper.lock_reward_condition.subscribe_condition_fulfilled(
        agreement_id,
        10,
        log_event(keeper.lock_reward_condition.FULFILLED_EVENT),
        (),
        wait=True
    )
    assert event, 'no event for LockRewardCondition.Fulfilled'
    assert keeper.condition_manager.get_condition_state(lock_cond_id) == 2, ''
    assert keeper.token.get_token_balance(
        keeper.escrow_reward_condition.address
    ) == (price + starting_balance), ''

    # Fulfill access_secret_store_condition
    tx_hash = keeper.access_secret_store_condition.fulfill(
        agreement_id, asset_id, consumer_acc.address, publisher_acc
    )
    keeper.access_secret_store_condition.get_tx_receipt(tx_hash)
    event = keeper.access_secret_store_condition.subscribe_condition_fulfilled(
        agreement_id,
        20,
        log_event(keeper.access_secret_store_condition.FULFILLED_EVENT),
        (),
        wait=True
    )
    assert event, 'no event for AccessSecretStoreCondition.Fulfilled'
    assert keeper.condition_manager.get_condition_state(access_cond_id) == 2, ''

    # Fulfill escrow_reward_condition
    tx_hash = keeper.escrow_reward_condition.fulfill(
        agreement_id, price, publisher_acc.address,
        consumer_acc.address, lock_cond_id,
        access_cond_id, publisher_acc
    )
    keeper.escrow_reward_condition.get_tx_receipt(tx_hash)
    event = keeper.escrow_reward_condition.subscribe_condition_fulfilled(
        agreement_id,
        10,
        log_event(keeper.escrow_reward_condition.FULFILLED_EVENT),
        (),
        wait=True
    )
    assert event, 'no event for EscrowReward.Fulfilled'
    assert keeper.condition_manager.get_condition_state(escrow_cond_id) == 2, ''
    assert keeper.token.get_token_balance(
        keeper.escrow_reward_condition.address
    ) == starting_balance, ''
    assert keeper.token.get_token_balance(publisher_acc.address) == (pub_token_balance + price), ''


@e2e_test
def test_agreement_hash():
    """
    This test verifies generating agreement hash using fixed inputs and ddo example.
    This will make it easier to compare the hash generated from different languages.
    """
    did = "did:op:0bc278fee025464f8012b811d1bce8e22094d0984e4e49139df5d5ff7a028bdf"
    template_id = '0x34219511357c43f4af90a4896b972a32809379bfbe8148eab3878e8406e75836'
    agreement_id = '0xf136d6fadecb48fdb2fc1fb420f5a5d1c32d22d9424e47ab9461556e058fefaa'
    ddo = get_ddo_sample()

    sa = ServiceAgreement.from_json(
        ddo.get_service(ServiceTypes.ASSET_ACCESS).as_dictionary())
    assert did == ddo.did
    # Don't generate condition ids, use fixed ids so we get consistent hash
    # (access_id, lock_id, escrow_id) = sa.generate_agreement_condition_ids(
    #     agreement_id, ddo.asset_id, consumer, publisher, keeper)
    access_id = '0x2d7c1d60dc0c3f52aa9bd71ffdbe434a0e58435571e64c893bc9646fea7f6ec1'
    lock_id = '0x1e265c434c14e668695dda1555088f0ea4356f596bdecb8058812e7dcba9ee73'
    escrow_id = '0xe487fa6d435c2f09ef14b65b34521302f1532ac82ba8f6c86116acd8566e2da3'
    print(f'condition ids: \n'
          f'{access_id} \n'
          f'{lock_id} \n'
          f'{escrow_id}')
    agreement_hash = ServiceAgreement.generate_service_agreement_hash(
        template_id,
        (access_id, lock_id, escrow_id),
        sa.conditions_timelocks,
        sa.conditions_timeouts,
        agreement_id,
        generate_multi_value_hash
    )
    print('agreement hash: ', agreement_hash.hex())
    expected = '0xc1bc6cdb161b9d04d5b48bceba51b9092c4da8db08003b96b93647fbe86e330c'
    print('expected hash: ', expected)
    assert agreement_hash.hex() == expected, 'hash does not match.'


def test_agreement():
    template_id = '0xda558ceb3abd482db04ea813810ae9c6cea5bf13ef9943908b1a8b51c4e56c7f'
    agreement_id = '0x' + ('e' * 64)
    access_id = '0x' + ('a' * 64)
    lock_id = '0x' + ('b' * 64)
    escrow_id = '0x' + ('c' * 64)

    signature = ServiceAgreement.generate_service_agreement_hash(
        template_id,
        [access_id, lock_id, escrow_id],
        [0, 0, 0],
        [0, 0, 0],
        agreement_id,
        generate_multi_value_hash
    )

    print({signature})
    assert (
        signature == Web3.toBytes(
            hexstr="0x3450be8127dbe156fad90956fd351df42ffcfabba89d230406a301ceb5b40f92"
        )), "The signature is not correct."



#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

from .tiers import should_run_test


def test_unit_tier():
    assert should_run_test('unit', 'unit') is True
    assert should_run_test('integration', 'unit') is False
    assert should_run_test('e2e', 'unit') is False


def test_integration_tier():
    assert should_run_test('unit', 'integration') is True
    assert should_run_test('integration', 'integration') is True
    assert should_run_test('e2e', 'integration') is False


def test_e2e_tier():
    assert should_run_test('unit', 'e2e') is True
    assert should_run_test('integration', 'e2e') is True
    assert should_run_test('e2e', 'e2e') is True



"""
    Public key base

    Currently this class stores the public keys in the same form as in the JSON
    text data.

"""

#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

import json
import re
from base64 import b64decode, b64encode, b85decode, b85encode

PUBLIC_KEY_STORE_TYPE_PEM = 'publicKeyPem'
PUBLIC_KEY_STORE_TYPE_JWK = 'publicKeyJwk'
PUBLIC_KEY_STORE_TYPE_HEX = 'publicKeyHex'
PUBLIC_KEY_STORE_TYPE_BASE64 = 'publicKeyBase64'
PUBLIC_KEY_STORE_TYPE_BASE85 = 'publicKeyBase85'


class PublicKeyBase:
    """Base Public Key class, to allow to perfom basic key storage and validation using DDO keys."""

    def __init__(self, key_id, **kwargs):
        self._id = key_id
        self._owner = kwargs.get('owner', None)
        self._type = kwargs.get('type', None)

    def get_id(self):
        """Get the key id."""
        return self._id

    def assign_did(self, did):
        """
        assign the DID as the key id, if the DID does not have a '#value'
        at the end, then automatically add a new key value
        """
        if re.match('^#.*', self._id):
            self._id = did + self._id
        if re.match('^#.*', self._owner):
            self._owner = did + self._owner

    def get_owner(self):
        """Get the owner of this key."""
        return self._owner

    # def set_owner(self, value):
    # self._owner = value

    def get_type(self):
        """Get the type of key."""
        return self._type

    def get_store_type(self):
        """Get the type of key storage."""
        return self._store_type

    def set_key_value(self, value, store_type=PUBLIC_KEY_STORE_TYPE_BASE64):
        """Set the key value based on it's storage type."""
        if isinstance(value, dict):
            if PUBLIC_KEY_STORE_TYPE_HEX in value:
                self.set_key_value(value[PUBLIC_KEY_STORE_TYPE_HEX], PUBLIC_KEY_STORE_TYPE_HEX)
            elif PUBLIC_KEY_STORE_TYPE_BASE64 in value:
                self.set_key_value(value[PUBLIC_KEY_STORE_TYPE_BASE64],
                                   PUBLIC_KEY_STORE_TYPE_BASE64)
            elif PUBLIC_KEY_STORE_TYPE_BASE85 in value:
                self.set_key_value(value[PUBLIC_KEY_STORE_TYPE_BASE85],
                                   PUBLIC_KEY_STORE_TYPE_BASE85)
            elif PUBLIC_KEY_STORE_TYPE_JWK in value:
                self.set_key_value(value[PUBLIC_KEY_STORE_TYPE_JWK], PUBLIC_KEY_STORE_TYPE_JWK)
            elif PUBLIC_KEY_STORE_TYPE_PEM in value:
                self.set_key_value(value[PUBLIC_KEY_STORE_TYPE_PEM], PUBLIC_KEY_STORE_TYPE_PEM)
        else:
            self._value = value
            self._store_type = store_type

    def set_encode_key_value(self, value, store_type):
        """Save the key value base on it's storage type."""
        self._store_type = store_type
        if store_type == PUBLIC_KEY_STORE_TYPE_HEX:
            self._value = value.hex()
        elif store_type == PUBLIC_KEY_STORE_TYPE_BASE64:
            self._value = b64encode(value).decode()
        elif store_type == PUBLIC_KEY_STORE_TYPE_BASE85:
            self._value = b85encode(value).decode()
        elif store_type == PUBLIC_KEY_STORE_TYPE_JWK:
            # TODO: need to decide on which jwk library to import?
            raise NotImplementedError
        else:
            self._value = value
        return value

    def get_decode_value(self):
        """Return the key value based on it's storage type."""
        if self._store_type == PUBLIC_KEY_STORE_TYPE_HEX:
            value = bytes.fromhex(self._value)
        elif self._store_type == PUBLIC_KEY_STORE_TYPE_BASE64:
            value = b64decode(self._value)
        elif self._store_type == PUBLIC_KEY_STORE_TYPE_BASE85:
            value = b85decode(self._value)
        elif self._store_type == PUBLIC_KEY_STORE_TYPE_JWK:
            # TODO: need to decide on which jwk library to import?
            raise NotImplementedError
        else:
            value = self._value
        return value

    def get_value(self):
        """Get the key value."""
        return self._value

    def as_text(self, is_pretty=False):
        """Return the key as JSON text."""
        values = {'id': self._id, 'type': self._type}
        if self._owner:
            values['owner'] = self._owner

        if is_pretty:
            return json.dumps(values, indent=4, separators=(',', ': '))

        return json.dumps(values)

    def as_dictionary(self):
        """Return the key as a python dictionary."""
        values = {
            'id': self._id,
            'type': self._type
        }

        if self._owner:
            values['owner'] = self._owner
        return values

    def is_valid(self):
        """Return True if the key structure is valid."""
        return self._id and self._type

    def get_authentication_type(self):
        """Base overloaded method to return the authentication type to use for this key."""
        raise NotImplementedError
